<?php


namespace FiloBlu\OrderFlowRectifier\Model\Rectifiers;

use FiloBlu\OrderFlowRectifier\Api\RectifierInterface;

class CompleteKlarnaPaymentsDetailsRectifier implements RectifierInterface
{
    const METHOD_CODE = 'method_code';
    protected $connection;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $connection
    )
    {
        $this->connection = $connection->getConnection();
    }

    public function getInfo()
    {
        return 'Update Klarna payments details with method code name';
    }

    public function getTitle()
    {
        return 'Complete Klarna Payments Details';
    }

    public function executeSingle($order)
    {
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');
        $orderTableGrid = $this->connection->getTableName('sales_order_grid');
        $payment_additional_information = json_decode($order->additional_information, true);
        $affectedRows = 0;

        if(!is_null($payment_additional_information) && isset($payment_additional_information[self::METHOD_CODE]))
        {
            $methodCode = $payment_additional_information[self::METHOD_CODE];
            $affectedRows = $this->connection->update(
                $orderPaymentTable,
                ['cc_type' => $methodCode],
                ['parent_id = ?' => $order->entity_id]);

            if($affectedRows > 0)
            {
                $affectedRows += $this->connection->update(
                    $orderTableGrid,
                    ['payment_cctype' => $methodCode],
                    ['increment_id = ?' => $order->increment_id]);
            }
        }

        return $affectedRows > 0;
    }

    public function loadOrders()
    {
        $orderTable = $this->connection->getTableName('sales_order');
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');

        $query = "SELECT so.entity_id, so.increment_id, sop.method, sop.additional_information
FROM {$orderTable} so
INNER JOIN {$orderPaymentTable} sop ON so.entity_id = sop.parent_id
WHERE
      sop.method IN ('klarna_kp') AND
      (sop.cc_type IS NULL OR sop.cc_type NOT LIKE '%klarna%')";

        $ordersToBeRectified = $this->connection->fetchAll($query, [], \Zend_Db::FETCH_OBJ);
        return $ordersToBeRectified;
    }
}
